﻿using System.IO;
using System.Text;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Microsoft.WindowsAzure.Storage;
using Microsoft.WindowsAzure.Storage.Blob;

namespace AppFx.DownloadStream.AzureBlob.Tests
{
    [TestClass]
    public class StreamedBlobDownloadTests
    {        
        private const string OutputFile = "Text.txt";
        private const string StorageConnection = "DefaultEndpointsProtocol=https;AccountName=[TBC];AccountKey=[TBC]";
        private const string StorageContainer = "testdatacontainer";
        private const string BlobName = "TestFile.txt";

        [TestInitialize]
        public void Setup()
        {
            var storageAccount = CloudStorageAccount.Parse(StorageConnection);
            var blobClient = storageAccount.CreateCloudBlobClient();
            var container = blobClient.GetContainerReference(StorageContainer);
            container.CreateIfNotExists(BlobContainerPublicAccessType.Off);

            var blockBlob = container.GetBlockBlobReference(BlobName);
            var stringBuilder = new StringBuilder();
            stringBuilder.Append('T', 100000);

            blockBlob.UploadText(stringBuilder.ToString());    
        }

        [TestCleanup]
        public void TearDown()
        {
            var storageAccount = CloudStorageAccount.Parse(StorageConnection);
            var blobClient = storageAccount.CreateCloudBlobClient();
            var container = blobClient.GetContainerReference(StorageContainer);            
            var blockBlob = container.GetBlockBlobReference(BlobName);
            blockBlob.Delete();
            container.Delete();
        }

        [TestMethod]
        public void StreamedAzureDownloadTest()
        {
            var blobDownloader = new BlobDownloader
            {
                BlobName = BlobName,
                BufferSize = 1024,
                StorageConnection = StorageConnection,
                StorageContainer = StorageContainer
            };
            blobDownloader.Open();
                       
            var downloadStream = new GenericDownloadStream(blobDownloader);            
            using (var fs = new FileStream(OutputFile, FileMode.Create, FileAccess.Write))
            {
                var index = 0;
                var charsRead = 0;
                while (true)
                {
                    byte[] buffer = new byte[1024];
                    charsRead = downloadStream.Read(buffer, index, 1024);
                    index = index + charsRead;
                    if (charsRead > 0)
                    {
                        fs.Write(buffer, 0, charsRead);
                        fs.Flush();
                    }
                    else
                        break;
                }
            } 
        }               
    }
}
