﻿using System;
using System.Diagnostics;
using System.IO;
using System.Runtime.CompilerServices;
using Microsoft.WindowsAzure.Storage;
using Microsoft.WindowsAzure.Storage.Blob;

namespace AppFx.DownloadStream.AzureBlob
{
    public class BlobDownloader : IDownload
    {
        public string StorageConnection { get; set; }        
        public string StorageContainer { get; set; }
        public string BlobName { get; set; }
        public int BufferSize { get; set; }        

        private CloudStorageAccount _storageAccount;
        private CloudBlobClient _blobClient;
        private CloudBlobContainer _container;
        private ICloudBlob _blob;
        private Stream _blobDownloadStream;
        private long _remainingBytes;
        private bool _firstRead;


        public void Open()
        {
            _storageAccount = CloudStorageAccount.Parse(StorageConnection);
            _blobClient = _storageAccount.CreateCloudBlobClient();
            _container = _blobClient.GetContainerReference(StorageContainer);
            _blob = _container.GetBlobReferenceFromServer(BlobName);
            _blobDownloadStream = _blob.OpenRead();
            _remainingBytes = 0;
            _firstRead = true;
        }

        public byte[] GetNext()
        {
            if (_firstRead)
            {
                _remainingBytes = _blobDownloadStream.Length;
                _firstRead = false;
            }
            if (_remainingBytes < BufferSize)
            {
                //Take a smaller chunk but all of whats left
                var buffer = new byte[_remainingBytes];
                _blobDownloadStream.Read(buffer, 0, (int)_remainingBytes);
                _remainingBytes = 0;
                return buffer;
            }
            if (_remainingBytes > BufferSize)
            {
                //Can take the whole chunk
                var buffer = new byte[BufferSize];
                _blobDownloadStream.Read(buffer, 0, BufferSize);
                _remainingBytes = _remainingBytes - BufferSize;
                return buffer;
            }
            return new byte[0];
        }
    }
}
